######################################################################
#  This file was made using the UWnineQuad2Dup problem type in GiD.  #
#  Created by:  Chris McGann and Pedro Arduino                       #
#              -University of Washington-                            #
######################################################################

wipe
*#
*# tcl procedures to designate the corner nodes
*tcl(makeCornerLayer)
*loop elems
*tcl(findCorners *ElemsConec(1) *ElemsConec(2) *ElemsConec(3) *ElemsConec(4))*\
*set var yEleSize(real) = operation(fabs(NodesCoord(3,2,real)-NodesCoord(1,2,real)))
*end elems
*#
#-----------------------------------------------------------------------------------------
#  1. CREATE PORE PRESSURE NODES AND FIXITIES
#-----------------------------------------------------------------------------------------
model BasicBuilder -ndm 2 -ndf 3

*# ... 3 dof node definition
# define pore pressure nodes:
*set Cond Nodal_DOF_on_nodes *nodes
*loop nodes
*if(cond(1,int)==3)
*format "%8i %10.3f %10.3f"
node *NodesNum *NodesCoord
*endif
*end nodes

set mNodeInfo [open nodeInfo.dat w]
*set Cond Nodal_DOF_on_nodes *nodes
*loop nodes
*if(cond(1,int)==3)
*format "%8i %10.3f %10.3f"
puts $mNodeInfo "*NodesNum *NodesCoord"
*endif
*end nodes
close $mNodeInfo
puts "Finished creating all -ndf 3 nodes..."

*# ... 3 dof boundary conditions and constraints
# define fixities for pore pressure nodes:
*set Layer cornerNodes *nodes
*loop nodes *OnlyInLayer
*set Cond 3_DOF_Point_Constraints_on_surfaces *nodes
*add Cond 3_DOF_Point_Constraints_on_lines *nodes
*add Cond 3_DOF_Point_Constraints_on_nodes *nodes
*if((cond(1,real)==1)||(cond(2,real)==1)||(cond(3,real)==1))
*format "%5i %2i %2i %2i"
fix *NodesNum *cond(1) *cond(2) *cond(3)
*endif
*end nodes
puts "Finished creating all -ndf 3 boundary conditions..."

# define equal degrees of freedom for pore pressure nodes
*set Layer cornerNodes *nodes
*loop nodes *OnlyInLayer
*set Cond EqualDOF_01_on_lines *nodes
*add Cond EqualDOF_01_on_nodes *nodes
*if(strcmp(cond(1),"master")==0)
*set var mstrNode(real) = NodesCoord(2,real)
equalDOF *NodesNum *\
*loop nodes *OnlyInLayer
*set var slvNode(real) = NodesCoord(2,real)
*set var differ(real) = operation(mstrNode-slvNode)
*if((strcmp(cond(1),"slave")==0)&&(fabs(differ)<0.0000001))
*NodesNum  1 2
*endif
*end nodes
*endif
*end nodes
*#
*set var eqDOF2(int) = 0
*set Cond EqualDOF_02_on_lines *nodes
*add Cond EqualDOF_02_on_nodes *nodes
*loop nodes *OnlyInCond
*if(strcmp(cond(1),"master")==0)
*set var eqDOF2(int) = 1
*endif
*end nodes
*if(eqDOF2(int)==1)

*set Layer cornerNodes *nodes
*loop nodes *OnlyInLayer
*set Cond EqualDOF_02_on_lines *nodes
*add Cond EqualDOF_02_on_nodes *nodes
*if(strcmp(cond(1),"master")==0)
*set var mstrNode(real) = NodesCoord(2,real)
equalDOF *NodesNum *\
*loop nodes *OnlyInLayer
*set var slvNode(real) = NodesCoord(2,real)
*set var differ(real) = operation(mstrNode-slvNode)
*if((strcmp(cond(1),"slave")==0)&&(fabs(differ)<0.0000001))
*NodesNum  1 2
*endif
*end nodes
*endif
*end nodes
*endif
puts "Finished creating equalDOF for pore pressure nodes..."

#-----------------------------------------------------------------------------------------
#  2. CREATE INTERIOR NODES AND FIXITIES
#-----------------------------------------------------------------------------------------
model BasicBuilder -ndm 2 -ndf 2

*# ... 2 dof node definition
# define interior nodes
*set Cond Nodal_DOF_on_nodes *nodes
*loop nodes
*if(cond(1,int)==2)
*format "%8i %10.3f %10.3f"
node *NodesNum *NodesCoord
*endif
*end nodes
puts "Finished creating all -ndf 2 nodes..."

*# create list of all nodes and locations 
*#set mNodesInfo [open allNodesInfo.dat w]
*#loop nodes
*#format "%8i %10.3f %10.3f"
*#puts $mNodesInfo "*NodesNum *NodesCoord"
*#end nodes
*#close $mNodesInfo

*# ... 2 dof boundary condtions and constraints
# define fixities for interior nodes:
*loop layers
*set Layer *LayerName *nodes
*remove Layer cornerNodes *nodes
*loop nodes *OnlyInLayer
*set Cond 2_DOF_Point_Constraints_on_lines *nodes
*add Cond 2_DOF_Point_Constraints_on_nodes *nodes
*if((cond(1,real)==1)||(cond(2,real)==1))
*format "%5i %2i %2i"
fix *NodesNum *cond(1) *cond(2)
*endif
*end nodes
*end layers
puts "Finished creating all -ndf 2 boundary conditions..."

# define equal degrees of freedom which have not yet been defined
*loop nodes 
*set Cond EqualDOF_01_on_lines *nodes
*add Cond EqualDOF_01_on_nodes *nodes
*if(strcmp(cond(1),"master")==0)
*set var mstrY(real) = NodesCoord(2,real)
*set var mstrNode(int) = NodesNum(int)
*loop nodes *OnlyInCond
*if(strcmp(NodesLayerName,"cornerNodes")!=0)
*set var slvY(real) = NodesCoord(2,real)
*set var diffY(real) = operation(mstrY-slvY)
*if((strcmp(cond(1),"slave")==0)&&(fabs(diffY)<0.0000001))
equalDOF *mstrNode *NodesNum  1 2
*endif
*endif
*end nodes
*endif
*end nodes
*set var eqDOF2(int) = 0
*set Cond EqualDOF_02_on_lines *nodes
*add Cond EqualDOF_02_on_nodes *nodes
*loop nodes *OnlyInCond
*if(strcmp(cond(1),"master")==0)
*set var eqDOF2(int) = 1
*endif
*end nodes
*if(eqDOF2(int)==1)

*loop nodes 
*set Cond EqualDOF_02_on_lines *nodes
*add Cond EqualDOF_02_on_nodes *nodes
*if(strcmp(cond(1),"master")==0)
*set var mstrY(real) = NodesCoord(2,real)
*set var mstrNode(int) = NodesNum(int)
*loop nodes *OnlyInCond
*if(strcmp(NodesLayerName,"cornerNodes")!=0)
*set var slvY(real) = NodesCoord(2,real)
*set var diffY(real) = operation(mstrY-slvY)
*if((strcmp(cond(1),"slave")==0)&&(fabs(diffY)<0.0000001))
equalDOF *mstrNode *NodesNum  1 2
*endif
*endif
*end nodes
*endif
*end nodes
*endif
puts "Finished creating equalDOF constraints..."

*# base equalDOF
*set Cond Dashpot_properties_and_slave_node *nodes
*loop nodes *OnlyInCond
*if(cond(1,int)==1)
*set var dNode(int) = NodesNum(int)
*endif
*end nodes
*set Cond EqualDOF_forBase_on_lines *nodes
*add Cond EqualDOF_forBase_on_nodes *nodes
*loop nodes *OnlyInCOnd
*if(cond(1,int)==1)
*if(NodesNum(int)==dNode)
*# do nothing
*elseif(NodesNum(int)!=dNode)
equalDOF *dNode *NodesNum  1
*endif
*endif
*end nodes
puts "Finished creating equalDOF for base..."

#-----------------------------------------------------------------------------------------
#  3. CREATE SOIL MATERIALS
#-----------------------------------------------------------------------------------------
*#
*# ... define preliminary parameters
*set var g(real)=Operation(9.81)
*set var matCount(int) = 0

*loop materials
*if(strcmp(MatProp(1),"ElasticIsotropic")==0)
nDMaterial ElasticIsotropic *matnum() *MatProp(3) *MatProp(4) *MatProp(2)
*set var xWgt(real) = operation(g*MatProp(8,real))
*set Var yWgt(real) = operation(-g*MatProp(9,real))
set thick*matnum() *MatProp(10)
*format "%1i %8.2f"
set xWgt*matnum() *xWgt
set yWgt*matnum() *yWgt
set uBulk*matnum() *MatProp(5)
set hPerm*matnum() *MatProp(6)
set vPerm*matnum() *MatProp(7)
*endif
*#
*if(strcmp(MatProp(1),"PressureDependMultiYield")==0)
nDMaterial PressureDependMultiYield *matnum() 2 *MatProp(2) *MatProp(3) *MatProp(4) *MatProp(5) *MatProp(6) \
                                    *MatProp(7) *MatProp(8) *MatProp(9) *MatProp(10) *MatProp(11) *MatProp(12) \
                                    *MatProp(13) *MatProp(14) *MatProp(15) *MatProp(16) *MatProp(17) \
                                    *MatProp(18) *MatProp(19) *MatProp(20) *MatProp(21)
*set var xWgt(real) = operation(g*MatProp(25,real))
*set Var yWgt(real) = operation(-g*MatProp(26,real))
set thick*matnum() *MatProp(27)
*format "%1i %8.2f"
set xWgt*matnum() *xWgt
set yWgt*matnum() *yWgt
set uBulk*matnum() *MatProp(22)
set hPerm*matnum() *MatProp(23)
set vPerm*matnum() *MatProp(24)
*endif
*#
*if(strcmp(MatProp(1),"PressureDependMultiYield02")==0)
nDMaterial PressureDependMultiYield02 *matnum() 2 *MatProp(2) *MatProp(3) *MatProp(4) *MatProp(5) *MatProp(6) \
                                      *MatProp(7) *MatProp(8) *MatProp(9) *MatProp(10) *MatProp(11) *MatProp(12) \
                                      *MatProp(13) *MatProp(14) *MatProp(15) *MatProp(16) *MatProp(17) \
                                      *MatProp(18) *MatProp(19) *MatProp(20) *MatProp(21) *MatProp(22) *MatProp(23)
*set var xWgt(real) = operation(g*MatProp(27,real))
*set Var yWgt(real) = operation(-g*MatProp(28,real))
set thick*matnum() *MatProp(29)
*format "%1i %8.2f"
set xWgt*matnum() *xWgt
set yWgt*matnum() *yWgt
set uBulk*matnum() *MatProp(24)
set hPerm*matnum() *MatProp(25)
set vPerm*matnum() *MatProp(26)
*endif
*#
*if(strcmp(MatProp(1),"PressureIndependMultiYield")==0)
nDMaterial PressureIndependMultiYield *matnum() 2 *MatProp(2) *MatProp(3) *MatProp(4) *MatProp(5) *MatProp(6) *\
*MatProp(7) *MatProp(8) *MatProp(9) *MatProp(10)
*set var xWgt(real) = operation(g*MatProp(14,real))
*set Var yWgt(real) = operation(-g*MatProp(15,real))
set thick*matnum() *MatProp(16)
*format "%1i %8.2f"
set xWgt*matnum() *xWgt
set yWgt*matnum() *yWgt
set uBulk*matnum() *MatProp(11)
set hPerm*matnum() *MatProp(12)
set vPerm*matnum() *MatProp(13)
*endif
*#
*if(strcmp(MatProp(1),"J2Plasticity")==0)
nDMaterial J2Plasticity *matnum() *matnum() 2 *MatProp(2) *MatProp(3) *MatProp(4) *MatProp(5) *MatProp(6) *MatProp(7) *MatProp(8)
*set var xWgt(real) = operation(g*MatProp(12,real))
*set Var yWgt(real) = operation(-g*MatProp(13,real))
set thick*matnum() *MatProp(14)
*format "%1i %8.2f"
set xWgt*matnum() *xWgt
set yWgt*matnum() *yWgt
set uBulk*matnum() *MatProp(9)
set hPerm*matnum() *MatProp(10)
set vPerm*matnum() *MatProp(11)
*endif
*end materials
puts "Finished creating all soil materials..."

#-----------------------------------------------------------------------------------------
#  4. CREATE SOIL ELEMENTS
#-----------------------------------------------------------------------------------------

# permeabilities are initial set at 1.0 m/s for gravity analysis, values are updated after gravity
*loop elems
element 9_4_QuadUP *ElemsNum *ElemsConec(4,int) *ElemsConec(1,int) *ElemsConec(2,int) *ElemsConec(3,int) *\
*ElemsConec(8,int) *ElemsConec(5,int) *ElemsConec(6,int) *ElemsConec(7,int) *ElemsConec(9,int)*\
 $thick*ElemsMat *ElemsMat $uBulk*ElemsMat 1.0 1.0 1.0 $xWgt*ElemsMat $yWgt*ElemsMat
*end elems
puts "Finished creating all soil elements..."

#-----------------------------------------------------------------------------------------
#  5. LYSMER DASHPOT
#-----------------------------------------------------------------------------------------
*#
*set var iNode = operation(npoin+100)
*set var jNode = operation(iNode+1)
*set Cond Dashpot_properties_and_slave_node *nodes
*loop nodes *OnlyInCond
*if(cond(1,int)==1)
*set var dNode(int) = NodesNum(int)
*set var xCoord(real) = NodesCoord(1,real)
*set var yCoord(real) = NodesCoord(2,real)
*set var C(real) = operation(cond(2,real)*cond(3,real))
*set var mRho(real) = cond(3,real)
*set var baseArea(real) = cond(4,real)
*endif
*end nodes

# define dashpot nodes
*format "%8i %10.3f %10.3f%"
node *iNode *xCoord *yCoord
*format "%8i %10.3f %10.3f%"
node *jNode *xCoord *yCoord

# define fixities for dashpot nodes
fix *iNode  1 1
fix *jNode  0 1

# define equal DOF for dashpot and base soil node
equalDOF *dNode *jNode  1
puts "Finished creating dashpot nodes and boundary conditions..."

# define dashpot material
*set var dMat = operation(nmats+1)
*format "%10.2f"
set baseArea *baseArea
*format "%10.2f"
set dashpotCoeff *C
uniaxialMaterial Viscous *dMat [expr $dashpotCoeff**$baseArea] 1

# define dashpot element
*set var dElem = operation(nelem+1)
element zeroLength *dElem *iNode *jNode -mat *dMat -dir 1
puts "Finished creating dashpot material and element..."

#-----------------------------------------------------------------------------------------
#  6. DEFINE NODAL MASSES FOR MODELING WATER
#-----------------------------------------------------------------------------------------

# define nodal masses for 3 dof nodes
model BasicBuilder -ndm 2 -ndf 3
*set Layer cornerNodes *nodes
*loop nodes *OnlyInLayer
*set Cond Nodal_mass_on_lines *nodes
*add Cond Nodal_mass_on_nodes *nodes
*if(cond(1,real)==1)
*set var h20dpth(real) = operation(cond(3,real)-NodesCoord(2,real))
*set var h20mass(real) = operation(h20dpth(real)*cond(2,real))
*set var xMass(real) = operation(cond(4,real)*h20mass)
*set var yMass(real) = operation(cond(5,real)*h20mass)
*format "%8i %10.3f %10.3f %10.3f"
mass *NodesNum *xMass *yMass 0.000
*endif
*end nodes
puts "Finished creating -ndf 3 nodal masses..."

# define nodal masses for 2 dof nodes
model BasicBuilder -ndm 2 -ndf 2
*loop layers
*set Layer *LayerName *nodes
*remove Layer cornerNodes *nodes
*loop nodes *OnlyInLayer
*set Cond Nodal_mass_on_lines *nodes
*add Cond Nodal_mass_on_nodes *nodes
*if(cond(1,real)==1)
*set var h20dpth(real) = operation(cond(3,real)-NodesCoord(2,real))
*set var h20mass(real) = operation(h20dpth(real)*cond(2,real))
*set var xMass(real) = operation(cond(4,real)*h20mass)
*set var yMass(real) = operation(cond(5,real)*h20mass)
*format "%8i %10.3f %10.3f"
mass *NodesNum *xMass *yMass
*endif
*end nodes
*end layers
puts "Finished creating -ndf 2 nodal masses..."

#-----------------------------------------------------------------------------------------
#  7. CREATE GRAVITY RECORDERS
#-----------------------------------------------------------------------------------------

# create list for pore pressure nodes
set nodeList3 {}
set channel [open "nodeInfo.dat" r]
set count 0;
foreach line [split [read -nonewline $channel] \n] {
set count [expr $count+1];
set lineData($count) $line
set nodeNumber [lindex $lineData($count) 0]
lappend nodeList3 $nodeNumber
}
close $channel

# record nodal displacment, acceleration, and porepressure
eval "recorder Node -file Gdisplacement.out -time -node $nodeList3 -dof 1 2  disp"
eval "recorder Node -file Gacceleration.out -time -node $nodeList3 -dof 1 2  accel"
eval "recorder Node -file GporePressure.out -time -node $nodeList3 -dof 3 vel"
# record elemental stress and strain
recorder Element -file Gstress1.out   -time  -eleRange 1 *nelem  material 1 stress
recorder Element -file Gstress2.out   -time  -eleRange 1 *nelem  material 2 stress
recorder Element -file Gstress3.out   -time  -eleRange 1 *nelem  material 3 stress
recorder Element -file Gstress4.out   -time  -eleRange 1 *nelem  material 4 stress
recorder Element -file Gstress5.out   -time  -eleRange 1 *nelem  material 5 stress
recorder Element -file Gstress6.out   -time  -eleRange 1 *nelem  material 6 stress
recorder Element -file Gstress7.out   -time  -eleRange 1 *nelem  material 7 stress
recorder Element -file Gstress8.out   -time  -eleRange 1 *nelem  material 8 stress
recorder Element -file Gstress9.out   -time  -eleRange 1 *nelem  material 9 stress
recorder Element -file Gstrain1.out   -time  -eleRange 1 *nelem  material 1 strain
recorder Element -file Gstrain2.out   -time  -eleRange 1 *nelem  material 2 strain
recorder Element -file Gstrain3.out   -time  -eleRange 1 *nelem  material 3 strain
recorder Element -file Gstrain4.out   -time  -eleRange 1 *nelem  material 4 strain
recorder Element -file Gstrain5.out   -time  -eleRange 1 *nelem  material 5 strain
recorder Element -file Gstrain6.out   -time  -eleRange 1 *nelem  material 6 strain
recorder Element -file Gstrain7.out   -time  -eleRange 1 *nelem  material 7 strain
recorder Element -file Gstrain8.out   -time  -eleRange 1 *nelem  material 8 strain
recorder Element -file Gstrain9.out   -time  -eleRange 1 *nelem  material 9 strain
puts "Finished creating gravity recorders..."

#-----------------------------------------------------------------------------------------
#  8. CREATE FILES FOR POSTPROCESSING IN GiD
#-----------------------------------------------------------------------------------------

set meshFile [open renameMe.flavia.msh w]
puts $meshFile "MESH 94quad dimension 2 ElemType Quadrilateral Nnode 4"
puts $meshFile "Coordinates"
puts $meshFile "#node_number   coord_x   coord_y"
*set Cond Nodal_DOF_on_nodes *nodes
*loop nodes
*if(cond(1,int)==3)
*format "%8i %10.3f %10.3f"
puts $meshFile "*NodesNum *NodesCoord"
*endif
*end nodes
puts $meshFile "end coordinates"
puts $meshFile "Elements"
puts $meshFile "# element   node1   node2   node3   node4"
*loop elems
*format "%8i %8i %8i %8i %8i"
puts $meshFile "*ElemsNum *ElemsConec(4) *ElemsConec(1) *ElemsConec(2) *ElemsConec(3)"
*end elems
puts $meshFile "end elements"
close $meshFile

set eleFile [open elementInfo.dat w]
*loop elems
*format "%8i %8i %8i %8i %8i"
puts $eleFile "*ElemsNum *ElemsConec(4) *ElemsConec(1) *ElemsConec(2) *ElemsConec(3)"
*end elems
close $eleFile

#-----------------------------------------------------------------------------------------
#  9. DEFINE ANALYSIS PARAMETERS
#-----------------------------------------------------------------------------------------

#---GROUND MOTION PARAMETERS
# time step in ground motion record
set motionDT     0.005
# number of steps in ground motion record
set motionSteps  7990

#---RAYLEIGH DAMPING PARAMETERS
set pi      3.141592654
# damping ratio
set damp    0.02
# lower frequency
set omega1  [expr 2**$pi**0.2]
# upper frequency
set omega2  [expr 2**$pi**20]
# damping coefficients
set a0      [expr 2**$damp**$omega1**$omega2/($omega1 + $omega2)]
set a1      [expr 2**$damp/($omega1 + $omega2)]
puts "damping coefficients: a_0 = $a0;  a_1 = $a1"

#---DETERMINE STABLE ANALYSIS TIME STEP USING CFL CONDITION
# maximum shear wave velocity (m/s)
set vsMax       250.0
# element size (m)
set eleSize *yEleSize
# duration of ground motion (s)
set duration    [expr $motionDT**$motionSteps]
# trial analysis time step
set kTrial      [expr $eleSize/(pow($vsMax,0.5))]
# define time step and number of steps for analysis
if { $motionDT <= $kTrial } {
    set nSteps  $motionSteps
    set dT      $motionDT
} else {
    set nSteps  [expr int(floor($duration/$kTrial)+1)]
    set dT      [expr $duration/$nSteps]
}
puts "number of steps in analysis: $nSteps"
puts "analysis time step: $dT"

#---ANALYSIS PARAMETERS
# Newmark parameters
set gamma  0.5
set beta   0.25

#-----------------------------------------------------------------------------------------
#  10. GRAVITY ANALYSIS
#-----------------------------------------------------------------------------------------

# update materials to ensure elastic behavior
*loop materials
updateMaterialStage -material *matnum() -stage 0
*end materials

constraints Penalty 1.e18 1.e18
test        NormDispIncr 1e-3 35 1
algorithm   Newton
numberer    RCM
system      ProfileSPD
integrator  Newmark $gamma $beta
analysis    Transient

set startT  [clock seconds]
analyze     10 5.0e2
analyze     10 5.0e3
puts "Finished with elastic gravity analysis..."

# update materials to consider plastic behavior
*loop materials
updateMaterialStage -material *matnum() -stage 1
*end materials

# plastic gravity loading
analyze     10 5.0e-3
puts "Finished with plastic gravity analysis..."

#-----------------------------------------------------------------------------------------
#  11. UPDATE ELEMENT PERMEABILITY VALUES FOR POST-GRAVITY ANALYSIS
#-----------------------------------------------------------------------------------------

# choose base number for parameter IDs which is higer than other tags used in analysis
set ctr 10000.0
# loop over elements to define parameter IDs
for {set i 1} {$i<=*nelem} {incr i 1} {
    parameter [expr int($ctr+1.0)] element $i vPerm
	parameter [expr int($ctr+2.0)] element $i hPerm
	set ctr [expr $ctr+2.0]
}

# update permeability parameters for each element
*set var ctr(int) = 10001
*loop elems
updateParameter *ctr $vPerm*ElemsMat
*set var ctr(int) = operation(ctr+1)
updateParameter *ctr $hPerm*ElemsMat
*set var ctr(int) = operation(ctr+1)
*end elems

#-----------------------------------------------------------------------------------------
#  12. CREATE POST-GRAVITY RECORDERS
#-----------------------------------------------------------------------------------------

# reset time and analysis
setTime 0.0
wipeAnalysis
remove recorders

# recorder time step
set recDT  [expr 2**$motionDT]

# record nodal displacment, acceleration, and porepressure
eval "recorder Node -file displacement.out -time -dT $recDT -node $nodeList3 -dof 1 2  disp"
eval "recorder Node -file acceleration.out -time -dT $recDT -node $nodeList3 -dof 1 2  accel"
eval "recorder Node -file porePressure.out -time -dT $recDT -node $nodeList3 -dof 3 vel"
# record elemental stress and strain
recorder Element -file stress1.out   -time -dT $recDT  -eleRange 1 *nelem  material 1 stress
recorder Element -file stress2.out   -time -dT $recDT  -eleRange 1 *nelem  material 2 stress
recorder Element -file stress3.out   -time -dT $recDT  -eleRange 1 *nelem  material 3 stress
recorder Element -file stress4.out   -time -dT $recDT  -eleRange 1 *nelem  material 4 stress
recorder Element -file stress5.out   -time -dT $recDT  -eleRange 1 *nelem  material 5 stress
recorder Element -file stress6.out   -time -dT $recDT  -eleRange 1 *nelem  material 6 stress
recorder Element -file stress7.out   -time -dT $recDT  -eleRange 1 *nelem  material 7 stress
recorder Element -file stress8.out   -time -dT $recDT  -eleRange 1 *nelem  material 8 stress
recorder Element -file stress9.out   -time -dT $recDT  -eleRange 1 *nelem  material 9 stress
recorder Element -file strain1.out   -time -dT $recDT  -eleRange 1 *nelem  material 1 strain
recorder Element -file strain2.out   -time -dT $recDT  -eleRange 1 *nelem  material 2 strain
recorder Element -file strain3.out   -time -dT $recDT  -eleRange 1 *nelem  material 3 strain
recorder Element -file strain4.out   -time -dT $recDT  -eleRange 1 *nelem  material 4 strain
recorder Element -file strain5.out   -time -dT $recDT  -eleRange 1 *nelem  material 5 strain
recorder Element -file strain6.out   -time -dT $recDT  -eleRange 1 *nelem  material 6 strain
recorder Element -file strain7.out   -time -dT $recDT  -eleRange 1 *nelem  material 7 strain
recorder Element -file strain8.out   -time -dT $recDT  -eleRange 1 *nelem  material 8 strain
recorder Element -file strain9.out   -time -dT $recDT  -eleRange 1 *nelem  material 9 strain
puts "Finished creating all recorders..."

#-----------------------------------------------------------------------------------------
#  13. DYNAMIC ANALYSIS
#-----------------------------------------------------------------------------------------

model BasicBuilder -ndm 2 -ndf 3

# define constant scaling factor for applied velocity
set cFactor [expr $baseArea**$dashpotCoeff]

# define velocity time history file
set velocityFile velocityHistory.out

# timeseries object for force history
set mSeries "Path -dt $motionDT -filePath $velocityFile -factor $cFactor"

# loading object
pattern Plain 10 $mSeries {
    load *dNode 1.0 0.0 0.0
}
puts "Dynamic loading created..."

constraints Penalty 1.e20 1.e20
test        NormDispIncr 1.0e-3 35 1
algorithm   Newton
numberer    RCM
system      ProfileSPD
integrator  Newmark $gamma $beta
rayleigh    $a0 $a1 0.0 0.0
analysis    Transient

# perform analysis with timestep reduction loop
set ok [analyze $nSteps  $dT]

# if analysis fails, reduce timestep and continue with analysis
if {$ok != 0} {
    puts "did not converge, reducing time step"
    set curTime  [getTime]
    puts "curTime: $curTime"
    set curStep  [expr $curTime/$dT]
    puts "curStep: $curStep"
    set remStep  [expr int(($nSteps-$curStep)**2.0)]
    puts "remStep: $remStep"
    set dT       [expr $dT/2.0]
    puts "dT: $dT"

    set ok [analyze  $remStep  $dT]

    # if analysis fails again, reduce timestep and continue with analysis
    if {$ok != 0} {
        puts "did not converge, reducing time step"
        set curTime  [getTime]
        puts "curTime: $curTime"
        set curStep  [expr $curTime/$dT]
        puts "curStep: $curStep"
        set remStep  [expr int(($remStep-$curStep)**2.0)]
        puts "remStep: $remStep"
        set dT       [expr $dT/2.0]
        puts "dT: $dT"

        analyze  $remStep  $dT
    }
}
set endT    [clock seconds]
puts "Finished with dynamic analysis..."
puts "Analysis execution time: [expr $endT-$startT] seconds"

wipe
