####################################################################
#  This file was made using the UWquad2D problem type in GiD.      #
#  Created by:  Chris McGann and Pedro Arduino, 2010               #
#              -University of Washington-                          #
####################################################################

wipe

#-----------------------------------------------------------------------------------------
#  1. CREATE SOIL NODES AND FIXITIES
#-----------------------------------------------------------------------------------------
model BasicBuilder -ndm 2 -ndf 2

# define soil nodes
*loop nodes
*format "%8i %10.3f %10.3f"
node *NodesNum *NodesCoord
*end nodes
puts "Finished creating all -ndf 2 nodes..."

# create list of all nodes and locations 
set mNodeInfo [open nodeInfo.dat w]
*loop nodes
*format "%8i %10.3f %10.3f"
puts $mNodeInfo "*NodesNum *NodesCoord"
*end nodes
close $mNodeInfo

# define fixities for soil nodes
*set Cond 2_DOF_Point_Constraints_on_lines *nodes
*add Cond 2_DOF_Point_Constraints_on_nodes *nodes
*loop nodes *OnlyInCond
*format "%5i %2i %2i"
fix *NodesNum *cond(1) *cond(2)
*end nodes
puts "Finished creating all -ndf 2 boundary conditions..."

# define equal degrees of freedom for free-field columns
*loop nodes 
*set Cond EqualDOF_01_on_lines *nodes
*add Cond EqualDOF_01_on_nodes *nodes
*if(strcmp(cond(1),"master")==0)
*set var mstrY(real) = NodesCoord(2,real)
*set var mstrNode(int) = NodesNum(int)
*loop nodes *OnlyInCond
*set var slvY(real) = NodesCoord(2,real)
*set var diffY(real) = operation(mstrY-slvY)
*if((strcmp(cond(1),"slave")==0)&&(fabs(diffY)<0.0000001))
equalDOF *mstrNode *NodesNum  1 2
*endif
*end nodes
*endif
*end nodes
*set var eqDOF2(int) = 0
*set Cond EqualDOF_02_on_lines *nodes
*add Cond EqualDOF_02_on_nodes *nodes
*loop nodes *OnlyInCond
*if(strcmp(cond(1),"master")==0)
*set var eqDOF2(int) = 1
*endif
*end nodes
*if(eqDOF2(int)==1)

*loop nodes 
*set Cond EqualDOF_02_on_lines *nodes
*add Cond EqualDOF_02_on_nodes *nodes
*if(strcmp(cond(1),"master")==0)
*set var mstrY(real) = NodesCoord(2,real)
*set var mstrNode(int) = NodesNum(int)
*loop nodes *OnlyInCond
*set var slvY(real) = NodesCoord(2,real)
*set var diffY(real) = operation(mstrY-slvY)
*if((strcmp(cond(1),"slave")==0)&&(fabs(diffY)<0.0000001))
equalDOF *mstrNode *NodesNum  1 2
*endif
*end nodes
*endif
*end nodes
*endif
puts "Finished creating equalDOF constraints for soil columns..."

*# base equalDOF
*set Cond Dashpot_properties_and_slave_node *nodes
*loop nodes *OnlyInCond
*if(cond(1,int)==1)
*set var dNode(int) = NodesNum(int)
*endif
*end nodes
*set Cond EqualDOF_forBase_on_lines *nodes
*add Cond EqualDOF_forBase_on_nodes *nodes
*loop nodes *OnlyInCOnd
*if(cond(1,int)==1)
*if(NodesNum(int)==dNode)
*# do nothing
*elseif(NodesNum(int)!=dNode)
equalDOF *dNode *NodesNum  1
*endif
*endif
*end nodes
puts "Finished creating equalDOF for base..."

#-----------------------------------------------------------------------------------------
#  2. CREATE SOIL MATERIALS
#-----------------------------------------------------------------------------------------
*#
*# ... define preliminary parameters
*set var g(real)=Operation(9.81)
*set var matCount(int) = 0

*loop materials
*if(strcmp(MatProp(1),"ElasticIsotropic")==0)
nDMaterial ElasticIsotropic *matnum() *MatProp(3) *MatProp(4) *MatProp(2)
*set var xWgt(real) = operation(g*MatProp(2,real)*MatProp(5,real))
*set Var yWgt(real) = operation(-g*MatProp(2,real)*MatProp(6,real))
set thick*matnum() *MatProp(7)
*format "%1i %8.2f"
set xWgt*matnum() *xWgt
set yWgt*matnum() *yWgt
*endif
*#
*if(strcmp(MatProp(1),"PressureDependMultiYield")==0)
nDMaterial PressureDependMultiYield *matnum() 2 *MatProp(2) *MatProp(3) *MatProp(4) *MatProp(5) *MatProp(6) \
                                    *MatProp(7) *MatProp(8) *MatProp(9) *MatProp(10) *MatProp(11) *MatProp(12) \
                                    *MatProp(13) *MatProp(14) *MatProp(15) *MatProp(16) *MatProp(17) \
                                    *MatProp(18) *MatProp(19) *MatProp(20) *MatProp(21)
*set var xWgt(real) = operation(g*MatProp(2,real)*MatProp(22,real))
*set Var yWgt(real) = operation(-g*MatProp(2,real)*MatProp(23,real))
set thick*matnum() *MatProp(24)
*format "%1i %8.2f"
set xWgt*matnum() *xWgt
set yWgt*matnum() *yWgt
*endif
*#
*if(strcmp(MatProp(1),"PressureDependMultiYield02")==0)
nDMaterial PressureDependMultiYield02 *matnum() 2 *MatProp(2) *MatProp(3) *MatProp(4) *MatProp(5) *MatProp(6) \
                                      *MatProp(7) *MatProp(8) *MatProp(9) *MatProp(10) *MatProp(11) *MatProp(12) \
                                      *MatProp(13) *MatProp(14) *MatProp(15) *MatProp(16) *MatProp(17) \
                                      *MatProp(18) *MatProp(19) *MatProp(20) *MatProp(21) *MatProp(22) *MatProp(23)
*set var xWgt(real) = operation(g*MatProp(2,real)*MatProp(24,real))
*set Var yWgt(real) = operation(-g*MatProp(2,real)*MatProp(25,real))
set thick*matnum() *MatProp(26)
*format "%1i %8.2f"
set xWgt*matnum() *xWgt
set yWgt*matnum() *yWgt
*endif
*#
*if(strcmp(MatProp(1),"PressureIndependMultiYield")==0)
nDMaterial PressureIndependMultiYield *matnum() 2 *MatProp(2) *MatProp(3) *MatProp(4) *MatProp(5) *MatProp(6) *\
*MatProp(7) *MatProp(8) *MatProp(9) *MatProp(10)
*set var xWgt(real) = operation(g*MatProp(2,real)*MatProp(11,real))
*set Var yWgt(real) = operation(-g*MatProp(2,real)*MatProp(12,real))
set thick*matnum() *MatProp(13)
*format "%1i %8.2f"
set xWgt*matnum() *xWgt
set yWgt*matnum() *yWgt
*endif
*#
*if(strcmp(MatProp(1),"J2Plasticity")==0)
nDMaterial J2Plasticity *matnum() *matnum() 2 *MatProp(2) *MatProp(3) *MatProp(4) *MatProp(5) *MatProp(6) *MatProp(7) *MatProp(8)
*set var xWgt(real) = operation(g*MatProp(2,real)*MatProp(9,real))
*set Var yWgt(real) = operation(-g*MatProp(2,real)*MatProp(10,real))
set thick*matnum() *MatProp(11)
*format "%1i %8.2f"
set xWgt*matnum() *xWgt
set yWgt*matnum() *yWgt
*endif
*end materials
puts "Finished creating all soil materials..."

#-----------------------------------------------------------------------------------------
#  3. CREATE SOIL ELEMENTS
#-----------------------------------------------------------------------------------------

# define soil elements
*loop elems
element quad *ElemsNum *ElemsConec(4,int) *ElemsConec(1,int) *ElemsConec(2,int) *ElemsConec(3,int) *\
 $thick*ElemsMat PlaneStrain *ElemsMat 0.0 0.0 $xWgt*ElemsMat $yWgt*ElemsMat
*end elems
puts "Finished creating all soil elements..."

# create list of element connectivities for post-processing
set eleFile [open elementInfo.dat w]
*loop elems
*format "%8i %8i %8i %8i %8i"
puts $eleFile "*ElemsNum *ElemsConec(4) *ElemsConec(1) *ElemsConec(2) *ElemsConec(3)"
*end elems
close $eleFile

#-----------------------------------------------------------------------------------------
#  4. LYSMER DASHPOT
#-----------------------------------------------------------------------------------------
*#
*set var iNode = operation(npoin+100)
*set var jNode = operation(iNode+1)
*set Cond Dashpot_properties_and_slave_node *nodes
*loop nodes *OnlyInCond
*if(cond(1,int)==1)
*set var dNode(int) = NodesNum(int)
*set var xCoord(real) = NodesCoord(1,real)
*set var yCoord(real) = NodesCoord(2,real)
*set var C(real) = operation(cond(2,real)*cond(3,real))
*set var mRho(real) = cond(3,real)
*set var baseArea(real) = cond(4,real)
*endif
*end nodes

# define dashpot nodes
*format "%8i %10.3f %10.3f%"
node *iNode *xCoord *yCoord
*format "%8i %10.3f %10.3f%"
node *jNode *xCoord *yCoord

# define fixities for dashpot nodes
fix *iNode  1 1
fix *jNode  0 1

# define equal DOF for dashpot and base soil node
equalDOF *dNode *jNode  1
puts "Finished creating dashpot nodes and boundary conditions..."

# define dashpot material
*set var dMat = operation(nmats+1)
*format "%10.2f"
set baseArea *baseArea
*format "%10.2f"
set dashpotCoeff *C
uniaxialMaterial Viscous *dMat [expr $dashpotCoeff**$baseArea] 1

# define dashpot element
*set var dElem = operation(nelem+1)
element zeroLength *dElem *iNode *jNode -mat *dMat -dir 1
puts "Finished creating dashpot material and element..."

#-----------------------------------------------------------------------------------------
#  5. DEFINE NODAL MASSES FOR MODELING WATER
#-----------------------------------------------------------------------------------------

# define nodal masses 
*set Cond Nodal_mass_on_lines *nodes
*add Cond Nodal_mass_on_nodes *nodes
*loop nodes *OnlyInCond
*set var h20dpth(real) = operation(cond(3,real)-NodesCoord(2,real))
*set var h20mass(real) = operation(h20dpth(real)*cond(2,real))
*set var xMass(real) = operation(cond(4,real)*h20mass)
*set var yMass(real) = operation(cond(5,real)*h20mass)
*format "%8i %10.3f %10.3f"
mass *NodesNum *xMass *yMass
*end nodes
puts "Finished creating -ndf 2 nodal masses..."

#-----------------------------------------------------------------------------------------
#  6. CREATE GRAVITY RECORDERS
#-----------------------------------------------------------------------------------------

# record nodal displacment, acceleration, and porepressure
recorder Node -file Gdisplacement.out -time -nodeRange 1 *npoin -dof 1 2  disp
recorder Node -file Gacceleration.out -time -nodeRange 1 *npoin -dof 1 2  accel
# record elemental stress and strain
recorder Element -file Gstress1.out   -time  -eleRange 1 *nelem  material 1 stress
recorder Element -file Gstress2.out   -time  -eleRange 1 *nelem  material 2 stress
recorder Element -file Gstress3.out   -time  -eleRange 1 *nelem  material 3 stress
recorder Element -file Gstress4.out   -time  -eleRange 1 *nelem  material 4 stress
recorder Element -file Gstrain1.out   -time  -eleRange 1 *nelem  material 1 strain
recorder Element -file Gstrain2.out   -time  -eleRange 1 *nelem  material 2 strain
recorder Element -file Gstrain3.out   -time  -eleRange 1 *nelem  material 3 strain
recorder Element -file Gstrain4.out   -time  -eleRange 1 *nelem  material 4 strain
puts "Finished creating gravity recorders..."

#-----------------------------------------------------------------------------------------
#  7. DEFINE ANALYSIS PARAMETERS
#-----------------------------------------------------------------------------------------

#---GROUND MOTION PARAMETERS
# time step in ground motion record
set motionDT     0.005
# number of steps in ground motion record
set motionSteps  7990

#---RAYLEIGH DAMPING PARAMETERS
set pi      3.141592654
# damping ratio
set damp    0.02
# lower frequency
set omega1  [expr 2**$pi**0.2]
# upper frequency
set omega2  [expr 2**$pi**20]
# damping coefficients
set a0      [expr 2**$damp**$omega1**$omega2/($omega1 + $omega2)]
set a1      [expr 2**$damp/($omega1 + $omega2)]
puts "damping coefficients: a_0 = $a0;  a_1 = $a1"

*#find minimum element size for use in defining time step
*set var yEleSize(real) = 100000.0
*loop elems
*set var curSize(real) = operation(fabs(NodesCoord(3,2,real)-NodesCoord(1,2,real)))
*if(curSize<yEleSize)
*set var yEleSize(real) = curSize
*endif
*end elems
#---DETERMINE STABLE ANALYSIS TIME STEP USING CFL CONDITION
# maximum shear wave velocity (m/s)
set vsMax       250.0
# element size (m)
set eleSize *yEleSize
# duration of ground motion (s)
set duration    [expr $motionDT**$motionSteps]
# trial analysis time step
set kTrial      [expr $eleSize/(pow($vsMax,0.5))]
# define time step and number of steps for analysis
if { $motionDT <= $kTrial } {
    set nSteps  $motionSteps
    set dT      $motionDT
} else {
    set nSteps  [expr int(floor($duration/$kTrial)+1)]
    set dT      [expr $duration/$nSteps]
}
puts "number of steps in analysis: $nSteps"
puts "analysis time step: $dT"

#---ANALYSIS PARAMETERS
# Newmark parameters
set gamma  0.5
set beta   0.25

#-----------------------------------------------------------------------------------------
#  8. GRAVITY ANALYSIS
#-----------------------------------------------------------------------------------------

# update materials to ensure elastic behavior
*loop materials
updateMaterialStage -material *matnum() -stage 0
*end materials

constraints Penalty 1.e18 1.e18
test        NormDispIncr 1e-3 35 1
algorithm   Newton
numberer    RCM
system      ProfileSPD
integrator  Newmark $gamma $beta
analysis    Transient

set startT  [clock seconds]
analyze     10 5.0e2
analyze     10 5.0e3
puts "Finished with elastic gravity analysis..."

# update materials to consider plastic behavior
*loop materials
updateMaterialStage -material *matnum() -stage 1
*end materials

# plastic gravity loading
analyze     10 5.0e-3
puts "Finished with plastic gravity analysis..."

#-----------------------------------------------------------------------------------------
#  9. CREATE POST-GRAVITY RECORDERS
#-----------------------------------------------------------------------------------------

# reset time and analysis
setTime 0.0
wipeAnalysis
remove recorders

# recorder time step
set recDT  [expr 2**$motionDT]

# record nodal displacment, acceleration, and porepressure
recorder Node -file displacement.out -time -dT $recDT -nodeRange 1 *npoin -dof 1 2  disp
recorder Node -file acceleration.out -time -dT $recDT -nodeRange 1 *npoin -dof 1 2  accel
# record elemental stress and strain
recorder Element -file stress1.out   -time -dT $recDT  -eleRange 1 *nelem  material 1 stress
recorder Element -file stress2.out   -time -dT $recDT  -eleRange 1 *nelem  material 2 stress
recorder Element -file stress3.out   -time -dT $recDT  -eleRange 1 *nelem  material 3 stress
recorder Element -file stress4.out   -time -dT $recDT  -eleRange 1 *nelem  material 4 stress
recorder Element -file strain1.out   -time -dT $recDT  -eleRange 1 *nelem  material 1 strain
recorder Element -file strain2.out   -time -dT $recDT  -eleRange 1 *nelem  material 2 strain
recorder Element -file strain3.out   -time -dT $recDT  -eleRange 1 *nelem  material 3 strain
recorder Element -file strain4.out   -time -dT $recDT  -eleRange 1 *nelem  material 4 strain
puts "Finished creating all recorders..."

#-----------------------------------------------------------------------------------------
#  10. DYNAMIC ANALYSIS
#-----------------------------------------------------------------------------------------

# define constant scaling factor for applied velocity
set cFactor [expr $baseArea**$dashpotCoeff]

# define velocity time history file
set velocityFile velocityHistory.out

# timeseries object for force history
set mSeries "Path -dt $motionDT -filePath $velocityFile -factor $cFactor"

# loading object
pattern Plain 10 $mSeries {
    load *dNode 1.0 0.0
}
puts "Dynamic loading created..."

constraints Penalty 1.e20 1.e20
test        NormDispIncr 1.0e-3 35 1
algorithm   Newton
numberer    RCM
system      ProfileSPD
integrator  Newmark $gamma $beta
rayleigh    $a0 $a1 0.0 0.0
analysis    Transient

# perform analysis with timestep reduction loop
set ok [analyze $nSteps  $dT]

# if analysis fails, reduce timestep and continue with analysis
if {$ok != 0} {
    puts "did not converge, reducing time step"
    set curTime  [getTime]
    puts "curTime: $curTime"
    set curStep  [expr $curTime/$dT]
    puts "curStep: $curStep"
    set remStep  [expr int(($nSteps-$curStep)**2.0)]
    puts "remStep: $remStep"
    set dT       [expr $dT/2.0]
    puts "dT: $dT"

    set ok [analyze  $remStep  $dT]

    # if analysis fails again, reduce timestep and continue with analysis
    if {$ok != 0} {
        puts "did not converge, reducing time step"
        set curTime  [getTime]
        puts "curTime: $curTime"
        set curStep  [expr $curTime/$dT]
        puts "curStep: $curStep"
        set remStep  [expr int(($remStep-$curStep)**2.0)]
        puts "remStep: $remStep"
        set dT       [expr $dT/2.0]
        puts "dT: $dT"

        analyze  $remStep  $dT
    }
}
set endT    [clock seconds]
puts "Finished with dynamic analysis..."
puts "Analysis execution time: [expr $endT-$startT] seconds"

wipe
